#!/usr/bin/python
#Anja, February 2017
#for python3.5
##########################################################################################
#Parses arCOG results to yield a tab-delimited file with one gene per line and listing corresponding arCOG and COG assignments as well as their descriptions, categories (class) in the folling format (if any):
#GeneID	arCOG	COG	class	description

#infile looks as following:

"""
Example of arCOG result file:
332798015,Acidianus_hospitalis_W1,332798015,103,1,103,arCOG00001,0,K,COG1695
302348728,Acidilobus_saccharovorans_345,302348728,126,1,126,arCOG00001,0,K,COG1695
302348796,Acidilobus_saccharovorans_345,302348796,340,246,340,arCOG00001,3,K,COG1695
302348989,Acidilobus_saccharovorans_345,302348989,132,1,132,arCOG00001,1,K,COG1695
432328170,Aciduliprofundum_MAR08_339,432328170,122,1,122,arCOG00001,0,

Example for description file:
arCOG   COG/Supercluster        FUNC    Gene    Annotation
arCOG00001      COG01695        K       -       Transcriptional regulator, PadR family
arCOG00002      COG01695        K       -       Transcriptional regulator, PadR family
arCOG00004      COG01695        K       -       Transcriptional regulator, PadR family
arCOG00005      COG01695        K       -       Transcriptional regulator, PadR family
"""




##########################################################################################
#Import modules
##########################################################################################

from Bio import SeqIO
import sys
import gzip
import os
import re
import argparse
from Bio.Alphabet import IUPAC



##########################################################################################
#ARGPARSE
##########################################################################################

parser = argparse.ArgumentParser(description='Parses arCOG results to yield a tab-delimited file with one gene per line and listing corresponding arCOG and COG assignments -if any- as well as their descriptions, and functional class in the folling format: GeneID	arCOG   COG class description')
parser.add_argument('-ac','--accessions', metavar='txt', required = True, help='protein accessions for genome of interest, one accession per line')
parser.add_argument('-a','--arcog_result', metavar='csv', required = True, help='File with arCOG assignments for all archaea, comma separated')
parser.add_argument('-d','--description', metavar='tsv', required = True, help='File with arCOG annotations from Kira Makarova')
parser.add_argument('-o','--output', metavar='tsv', help='proteins with corresponding arCOG information', required = True)
args = parser.parse_args()

if args.accessions is None and not os.path.exists(args.input):
    print("accessions file missing!")
    sys.exit(-1)

if args.arcog_result is None and not os.path.exists(args.input):
    print("arcog_result file missing!")
    sys.exit(-1)

if args.description is None and not os.path.exists(args.input):
    print("description file missing!")
    sys.exit(-1)



##########################################################################################
#Open and read files
##########################################################################################

proteinIDs = open(args.accessions).read().splitlines()
arCOG_file = open(args.arcog_result).readlines()
desc_file = open(args.description).readlines()
outfile = open("%s" % args.output,'w')



##########################################################################################
#Functions
##########################################################################################


#Read genes with corresponding arCOG domains into dictionary
##########################################################################################

def Read_arcog_results(arcogs):
    arcog_dict = {}
    for line in arcogs:
        entry = line.split(',')
        if 'arCOG' in entry[6]:
            if entry[0] not in arcog_dict:
                arcog_dict[entry[0]] = []
                arcog_dict[entry[0]].append(entry[6])
            else:
                arcog_dict[entry[0]].append(entry[6])

        else:
            pass
    return arcog_dict

#NOTICE: will list all arCOGs assigned to a specific gene; also if the same arCOG was assigned more than once to the same gene)



# Extract only those arCOGs, that correspond to accessions from genome of interest
##########################################################################################

def Get_arcog_for_acc(acclist, arcog_dict):
    acc_arCOG_dict = {}

    for acc in acclist:
        if acc in arcog_dict:
            acc_arCOG_dict[acc] = arcog_dict[acc]
        else:
            acc_arCOG_dict[acc] = "-"

    return acc_arCOG_dict




#Read mapping file with arCOG descriptions into dictionary
##########################################################################################

def Read_desc_file(descriptions):
    cog_dict = {}
    cat_dict = {}
    gene_dict = {}
    des_dict = {}

    for line in descriptions:
        entry = line.split('\t')
        if entry[1]:
            cog_dict[entry[0]] = entry[1].strip()
        else:
            cog_dict[entry[0]] = "-"
        if entry[2]:
            cat_dict[entry[0]] = entry[2].strip()
        else:
            cat_dict[entry[0]] = "-"
        if entry[3]:
            gene_dict[entry[0]] = entry[3].strip()
        else:
            gene_dict[entry[0]] = "-"
        if entry[4]:
            des_dict[entry[0]] = entry[4].strip()
        else:
            des_dict[entry[0]] = "-"

    return cog_dict, cat_dict, gene_dict, des_dict




##########################################################################################
#Parse all information into final table:
#GeneID	ArCOGs   COGs FUNCs    Genes    Annotations
##########################################################################################


#Dictionaries
acc_arcog_dict = Get_arcog_for_acc(proteinIDs, Read_arcog_results(arCOG_file))
cog_dict, cat_dict, gene_dict, des_dict = Read_desc_file(desc_file)
#print(acc_arcog_dict)

for acc in acc_arcog_dict:
    coglist = []
    catlist = []
    genelist = []
    deslist = []

    for arcog_entry in acc_arcog_dict[acc]:

        if acc_arcog_dict[acc] == "-":
            arCOG = "-"
            cog = "-"
            cat = "-"
            gen = "-"
            des = "-"

        else:
            if arcog_entry in cog_dict:
                coglist.append(cog_dict[arcog_entry])
            else:
                coglist.append("-")

            if arcog_entry in cat_dict:
                catlist.append(cat_dict[arcog_entry])
            else:
                catlist.append("-")

            if arcog_entry in gene_dict:
                genelist.append(gene_dict[arcog_entry])
            else:
                genelist.append("-")

            if arcog_entry in des_dict:
                deslist.append(des_dict[arcog_entry])
            else:
                deslist.append("-")

            arCOG = '; '.join(acc_arcog_dict[acc])
            cog = '; '.join(coglist)
            cat = '; '.join(catlist)
            gen = '; '.join(genelist)
            des = '; '.join(deslist)


    outfile.write("%s\t%s\t%s\t%s\t%s\t%s\n" % (acc, arCOG, cog, cat, gen, des))


outfile.close()
